@@ -183,23 +183,11 @@ static int mgs_create_ocsp_request(server_rec *s,
         return ret;
     }
 
-    /* issuer is set to a reference, so musn't be cleaned up */
-    gnutls_x509_crt_t issuer;
-    ret = gnutls_x509_trust_list_get_issuer(*req_data->trust, req_data->cert,
-                                            &issuer, 0);
-    if (ret != GNUTLS_E_SUCCESS)
-    {
-        ap_log_error(APLOG_MARK, APLOG_ERR, APR_EGENERAL, s,
-                     "Could not get issuer from trust list: %s (%d)",
-                     gnutls_strerror(ret), ret);
-        gnutls_ocsp_req_deinit(r);
-        return ret;
-    }
-
-    /* GnuTLS doc says that the digest is "normally"
-     * GNUTLS_DIG_SHA1. */
-    ret = gnutls_ocsp_req_add_cert(r, GNUTLS_DIG_SHA256,
-                                   issuer, req_data->cert);
+    /* Use SHA1 for issuer name hash and issuer key hash, for
+     * compliance with "lightweight" OCSP profile specified in RFC
+     * 5019. */
+    ret = gnutls_ocsp_req_add_cert(r, GNUTLS_DIG_SHA1,
+                                   req_data->issuer, req_data->cert);
 
     if (ret != GNUTLS_E_SUCCESS)
     {
@@ -286,14 +274,6 @@ int check_ocsp_response(server_rec *s, mgs_ocsp_data_t req_data,
     mgs_srvconf_rec *sc = (mgs_srvconf_rec *)
         ap_get_module_config(s->module_config, &gnutls_module);
 
-    if (req_data->trust == NULL)
-    {
-        ap_log_error(APLOG_MARK, APLOG_ERR, APR_EGENERAL, s,
-                     "No OCSP trust list available for server \"%s\"!",
-                     s->server_hostname);
-        return GNUTLS_E_NO_CERTIFICATE_FOUND;
-    }
-
     gnutls_ocsp_resp_t resp;
     int ret = gnutls_ocsp_resp_init(&resp);
     if (ret != GNUTLS_E_SUCCESS)
@@ -322,7 +302,7 @@ int check_ocsp_response(server_rec *s, mgs_ocsp_data_t req_data,
     }
 
     unsigned int verify;
-    ret = gnutls_ocsp_resp_verify(resp, *(req_data->trust), &verify, 0);
+    ret = gnutls_ocsp_resp_verify_direct(resp, req_data->issuer, &verify, 0);
     if (ret != GNUTLS_E_SUCCESS)
     {
         ap_log_error(APLOG_MARK, APLOG_ERR, APR_EGENERAL, s,
@@ -835,7 +815,7 @@ int mgs_get_ocsp_response(mgs_handle_t *ctxt,
     ocsp_response->size = OCSP_RESP_SIZE_MAX;
 
     apr_status_t rv = mgs_cache_fetch(sc->ocsp_cache,
-                                      ctxt->c->base_server,
+                                      sc->s,
                                       req_data->fingerprint,
                                       ocsp_response,
                                       ctxt->c->pool);
@@ -848,7 +828,7 @@ int mgs_get_ocsp_response(mgs_handle_t *ctxt,
     {
         ap_log_cerror(APLOG_MARK, APLOG_DEBUG, APR_EGENERAL, ctxt->c,
                       "Cached OCSP failure found for %s.",
-                      ctxt->c->base_server->server_hostname);
+                      sc->s->server_hostname);
         goto fail_cleanup;
     }
     else
@@ -872,7 +852,7 @@ int mgs_get_ocsp_response(mgs_handle_t *ctxt,
          * moment there's no good way to integrate that with the
          * Apache Mutex directive. */
         rv = mgs_cache_fetch(sc->ocsp_cache,
-                             ctxt->c->base_server,
+                             sc->s,
                              req_data->fingerprint,
                              ocsp_response,
                              ctxt->c->pool);
@@ -884,7 +864,7 @@ int mgs_get_ocsp_response(mgs_handle_t *ctxt,
             {
                 ap_log_cerror(APLOG_MARK, APLOG_DEBUG, APR_EGENERAL, ctxt->c,
                               "Cached OCSP failure found for %s.",
-                              ctxt->c->base_server->server_hostname);
+                              sc->s->server_hostname);
                 goto fail_cleanup;
             }
             else
@@ -897,13 +877,13 @@ int mgs_get_ocsp_response(mgs_handle_t *ctxt,
         }
     }
 
-    rv = mgs_cache_ocsp_response(ctxt->c->base_server, req_data, NULL);
+    rv = mgs_cache_ocsp_response(sc->s, req_data, NULL);
     if (rv != APR_SUCCESS)
     {
         ap_log_cerror(APLOG_MARK, APLOG_ERR, rv, ctxt->c,
                       "Caching a fresh OCSP response failed");
         /* cache failure to rate limit retries */
-        mgs_cache_ocsp_failure(ctxt->c->base_server,
+        mgs_cache_ocsp_failure(sc->s,
                                req_data,
                                sc->ocsp_failure_timeout);
         apr_global_mutex_unlock(sc->ocsp_mutex);
@@ -913,7 +893,7 @@ int mgs_get_ocsp_response(mgs_handle_t *ctxt,
 
     /* retry reading from cache */
     rv = mgs_cache_fetch(sc->ocsp_cache,
-                         ctxt->c->base_server,
+                         sc->s,
                          req_data->fingerprint,
                          ocsp_response,
                          ctxt->c->pool);
@@ -937,37 +917,6 @@ int mgs_get_ocsp_response(mgs_handle_t *ctxt,
 
 
 
-int mgs_create_ocsp_trust_list(gnutls_x509_trust_list_t *tl,
-                               const gnutls_x509_crt_t *chain,
-                               const int num)
-{
-    int added = 0;
-    int ret = gnutls_x509_trust_list_init(tl, num);
-
-    if (ret == GNUTLS_E_SUCCESS)
-        added = gnutls_x509_trust_list_add_cas(*tl, chain, num, 0);
-
-    if (added != num)
-        ret = GNUTLS_E_CERTIFICATE_ERROR;
-
-    /* Clean up trust list in case of error */
-    if (ret != GNUTLS_E_SUCCESS)
-        gnutls_x509_trust_list_deinit(*tl, 0);
-
-    return ret;
-}
-
-
-
-apr_status_t mgs_cleanup_trust_list(void *data)
-{
-    gnutls_x509_trust_list_t *tl = (gnutls_x509_trust_list_t *) data;
-    gnutls_x509_trust_list_deinit(*tl, 0);
-    return APR_SUCCESS;
-}
-
-
-
 apr_uri_t * mgs_cert_get_ocsp_uri(apr_pool_t *p, gnutls_x509_crt_t cert)
 {
     apr_pool_t *tmp;
@@ -997,7 +946,11 @@ apr_uri_t * mgs_cert_get_ocsp_uri(apr_pool_t *p, gnutls_x509_crt_t cert)
             ocsp_uri = apr_palloc(p, sizeof(apr_uri_t));
             rv = apr_uri_parse(p, ocsp_str, ocsp_uri);
             if (rv == APR_SUCCESS)
+            {
+                if (ocsp_uri->path == NULL)
+                    ocsp_uri->path = "/";
                 break;
+            }
             else
                 ocsp_uri = NULL;
         }
@@ -1124,10 +1077,8 @@ static apr_status_t mgs_async_ocsp_update(int state,
         ocsp_response.data = apr_palloc(pool, OCSP_RESP_SIZE_MAX);
         ocsp_response.size = OCSP_RESP_SIZE_MAX;
 
-        apr_status_t rv = mgs_cache_fetch(sc->ocsp_cache, server,
-                                          ocsp_data->fingerprint,
-                                          &ocsp_response,
-                                          pool);
+        rv = mgs_cache_fetch(sc->ocsp_cache, server, ocsp_data->fingerprint,
+                             &ocsp_response, pool);
 
         if (rv != APR_SUCCESS || (IS_FAILURE_RESPONSE(&ocsp_response)))
         {
@@ -1170,23 +1121,7 @@ static const char* configure_cert_staple(mgs_ocsp_data_t ocsp,
     if (ocsp->fingerprint.data == NULL)
         return "Could not read fingerprint from certificate!";
 
-    ocsp->trust = apr_palloc(pconf,
-                             sizeof(gnutls_x509_trust_list_t));
-    /* Only the direct issuer may sign the OCSP response or an
-     * OCSP signer. */
-    int ret = mgs_create_ocsp_trust_list(
-        ocsp->trust, &(sc->certs_x509_crt_chain[idx + 1]), 1);
-    if (ret != GNUTLS_E_SUCCESS)
-    {
-        ap_log_error(APLOG_MARK, APLOG_ERR, APR_EGENERAL, server,
-                     "Could not create OCSP trust list: %s (%d)",
-                     gnutls_strerror(ret), ret);
-        return "Could not build trust list for OCSP stapling!";
-    }
-    /* deinit trust list when the config pool is destroyed */
-    apr_pool_cleanup_register(pconf, ocsp->trust,
-                              mgs_cleanup_trust_list,
-                              apr_pool_cleanup_null);
+    ocsp->issuer = sc->certs_x509_crt_chain[idx + 1];
     return NULL;
 }
 
